@@ -34,6 +34,7 @@ typedef enum {
 	CommandInfo,
 	ExternalInfo,
 	PipeInfo,
+	PLPipeInfo,
 	DirectoryInfo,
 	WSMenuInfo,
 	WWindowListInfo,
@@ -82,6 +83,10 @@ typedef struct _Panel {
 	WMTextField *pipeT;
 	WMButton *pipeCacheB;
 
+	WMFrame *plpipeF;
+	WMTextField *plpipeT;
+	WMButton *plpipeCacheB;
+
 	WMFrame *dpathF;
 	WMTextField *dpathT;
 
@@ -174,7 +179,6 @@ static Bool shouldRemoveItem(struct WEditMenuDelegate *delegate, WEditMenu * men
 
 static void freeItemData(ItemData * data);
 
-extern char *capture_shortcut(Display *dpy, Bool *capturing, Bool convert_case);
 
 static WEditMenuDelegate menuDelegate = {
 	NULL,
@@ -293,7 +297,7 @@ static void changedItemPad(WMWidget * w, void *data)
 	panel->currentPad = padn;
 }
 
-static WEditMenu *putNewSubmenu(WEditMenu * menu, char *title)
+static WEditMenu *putNewSubmenu(WEditMenu * menu, const char *title)
 {
 	WEditMenu *tmp;
 	WEditMenuItem *item;
@@ -308,7 +312,7 @@ static WEditMenu *putNewSubmenu(WEditMenu * menu, char *title)
 	return tmp;
 }
 
-static ItemData *putNewItem(_Panel * panel, WEditMenu * menu, int type, char *title)
+static ItemData *putNewItem(_Panel * panel, WEditMenu * menu, int type, const char *title)
 {
 	WEditMenuItem *item;
 	ItemData *data;
@@ -391,6 +395,7 @@ static void createPanel(_Panel * p)
 
 		panel->markerPix[ExternalInfo] = pixm;
 		panel->markerPix[PipeInfo] = pixm;
+		panel->markerPix[PLPipeInfo] = pixm;
 		panel->markerPix[DirectoryInfo] = pixm;
 		panel->markerPix[WSMenuInfo] = pixm;
 		panel->markerPix[WWindowListInfo] = pixm;
@@ -424,6 +429,7 @@ static void createPanel(_Panel * p)
 		putNewSubmenu(pad, _("Submenu"));
 		putNewItem(panel, pad, ExternalInfo, _("External Submenu"));
 		putNewItem(panel, pad, PipeInfo, _("Generated Submenu"));
+		putNewItem(panel, pad, PLPipeInfo, _("Generated PL Menu"));
 		putNewItem(panel, pad, DirectoryInfo, _("Directory Contents"));
 		putNewItem(panel, pad, WSMenuInfo, _("Workspace Menu"));
 		putNewItem(panel, pad, WWindowListInfo, _("Window List Menu"));
@@ -642,6 +648,31 @@ static void createPanel(_Panel * p)
 
 	WMMapSubwidgets(panel->pipeF);
 
+	/* proplist pipe */
+
+	panel->plpipeF = WMCreateFrame(panel->optionsF);
+	WMResizeWidget(panel->plpipeF, width, 155);
+	WMMoveWidget(panel->plpipeF, 10, 30);
+	WMSetFrameTitle(panel->plpipeF, _("Command"));
+
+	panel->plpipeT = WMCreateTextField(panel->plpipeF);
+	WMResizeWidget(panel->plpipeT, width - 20, 20);
+	WMMoveWidget(panel->plpipeT, 10, 20);
+
+	WMAddNotificationObserver(dataChanged, panel, WMTextDidChangeNotification, panel->plpipeT);
+
+	label = WMCreateLabel(panel->plpipeF);
+	WMResizeWidget(label, width - 20, 40);
+	WMMoveWidget(label, 10, 50);
+	WMSetLabelText(label, _("Enter a command that outputs a proplist menu\n" "definition to stdout when invoked."));
+
+	panel->plpipeCacheB = WMCreateSwitchButton(panel->plpipeF);
+	WMResizeWidget(panel->plpipeCacheB, width - 20, 40);
+	WMMoveWidget(panel->plpipeCacheB, 10, 110);
+	WMSetButtonText(panel->plpipeCacheB, _("Cache menu contents after opening for\n" "the first time"));
+
+	WMMapSubwidgets(panel->plpipeF);
+
 	/* directory menu */
 
 	panel->dcommandF = WMCreateFrame(panel->optionsF);
@@ -790,6 +821,8 @@ static void createPanel(_Panel * p)
 
 	panel->sections[PipeInfo][0] = panel->pipeF;
 
+	panel->sections[PLPipeInfo][0] = panel->plpipeF;
+
 	panel->sections[DirectoryInfo][0] = panel->dpathF;
 	panel->sections[DirectoryInfo][1] = panel->dcommandF;
 
@@ -835,6 +868,10 @@ static void freeItemData(ItemData * data)
 		CFREE(data->param.pipe.command);
 		break;
 
+	case PLPipeInfo:
+		CFREE(data->param.pipe.command);
+		break;
+
 	case ExternalInfo:
 		CFREE(data->param.external.path);
 		break;
@@ -941,6 +978,22 @@ static ItemData *parseCommand(WMPropList * item)
 				data->param.external.path = p;
 			}
 		}
+	} else if (strcmp(command, "OPEN_PLMENU") == 0) {
+		char *p;
+
+		p = parameter;
+		while (isspace(*p) && *p)
+			p++;
+		if (*p == '|') {
+			if (*(p + 1) == '|') {
+				p++;
+				data->param.pipe.cached = 0;
+			} else {
+				data->param.pipe.cached = 1;
+			}
+			data->type = PLPipeInfo;
+			data->param.pipe.command = wtrimspace(p + 1);
+		}
 	} else if (strcmp(command, "WORKSPACE_MENU") == 0) {
 		data->type = WSMenuInfo;
 	} else if (strcmp(command, "WINDOWS_MENU") == 0) {
@@ -992,7 +1045,7 @@ static ItemData *parseCommand(WMPropList * item)
 	return data;
 }
 
-static void updateFrameTitle(_Panel * panel, char *title, InfoType type)
+static void updateFrameTitle(_Panel * panel, const char *title, InfoType type)
 {
 	if (type != NoInfo) {
 		char *tmp;
@@ -1014,6 +1067,10 @@ static void updateFrameTitle(_Panel * panel, char *title, InfoType type)
 			tmp = wstrconcat(title, _(": Program Generated Submenu"));
 			break;
 
+		case PLPipeInfo:
+			tmp = wstrconcat(title, _(": Program Generated Proplist Submenu"));
+			break;
+
 		case DirectoryInfo:
 			tmp = wstrconcat(title, _(": Directory Contents Menu"));
 			break;
@@ -1037,7 +1094,7 @@ static void updateFrameTitle(_Panel * panel, char *title, InfoType type)
 	}
 }
 
-static void changeInfoType(_Panel * panel, char *title, InfoType type)
+static void changeInfoType(_Panel * panel, const char *title, InfoType type)
 {
 	WMWidget **w;
 
@@ -1117,6 +1174,15 @@ static void updateMenuItem(_Panel * panel, WEditMenuItem * item, WMWidget * chan
 		}
 		break;
 
+	case PLPipeInfo:
+		if (changedWidget == panel->plpipeT) {
+			REPLACE(data->param.pipe.command, WMGetTextFieldText(panel->plpipeT));
+		}
+		if (changedWidget == panel->plpipeCacheB) {
+			data->param.pipe.cached = WMGetButtonSelected(panel->plpipeCacheB);
+		}
+		break;
+
 	case ExternalInfo:
 		if (changedWidget == panel->pathT) {
 			REPLACE(data->param.external.path, WMGetTextFieldText(panel->pathT));
@@ -1175,6 +1241,11 @@ menuItemCloned(WEditMenuDelegate * delegate, WEditMenu * menu, WEditMenuItem * o
 		newData->param.pipe.cached = data->param.pipe.cached;
 		break;
 
+	case PLPipeInfo:
+		newData->param.pipe.command = DUP(data->param.pipe.command);
+		newData->param.pipe.cached = data->param.pipe.cached;
+		break;
+
 	case ExternalInfo:
 		newData->param.external.path = DUP(data->param.external.path);
 		break;
@@ -1284,6 +1355,11 @@ static void menuItemSelected(WEditMenuDelegate * delegate, WEditMenu * menu, WEd
 			WMSetButtonSelected(panel->pipeCacheB, data->param.pipe.cached);
 			break;
 
+		case PLPipeInfo:
+			WMSetTextFieldText(panel->plpipeT, data->param.pipe.command);
+			WMSetButtonSelected(panel->plpipeCacheB, data->param.pipe.cached);
+			break;
+
 		case ExternalInfo:
 			WMSetTextFieldText(panel->pathT, data->param.external.path);
 			break;
@@ -1373,15 +1449,9 @@ static void buildMenuFromPL(_Panel * panel, WMPropList * pl)
 static WMPropList *getDefaultMenu(_Panel * panel)
 {
 	WMPropList *menu;
-	char *menuPath, *gspath;
-
-	gspath = wstrdup(WMAKER_RESOURCE_PATH);
-
-	menuPath = wmalloc(strlen(gspath) + 128);
-	sprintf(menuPath, "%s/plmenu", gspath);
+	static const char menuPath[] = WMAKER_RESOURCE_PATH "/plmenu";
 
 	menu = WMReadPropListFromFile(menuPath);
-
 	if (!menu) {
 		char *buffer, *msg;
 
@@ -1393,15 +1463,12 @@ static WMPropList *getDefaultMenu(_Panel * panel)
 		wfree(buffer);
 	}
 
-	wfree(gspath);
-	wfree(menuPath);
-
 	return menu;
 }
 
 static void showData(_Panel * panel)
 {
-	char *gspath;
+	const char *gspath;
 	char *menuPath;
 	WMPropList *pmenu;
 
@@ -1442,7 +1509,7 @@ static void showData(_Panel * panel)
 	WMReleasePropList(pmenu);
 }
 
-static Bool notblank(char *s)
+static Bool notblank(const char *s)
 {
 	if (s) {
 		while (*s++) {
@@ -1453,12 +1520,13 @@ static Bool notblank(char *s)
 	return False;
 }
 
-static WMPropList *processData(char *title, ItemData * data)
+static WMPropList *processData(const char *title, ItemData * data)
 {
 	WMPropList *item;
 	char *s1;
 	static WMPropList *pscut = NULL;
 	static WMPropList *pomenu = NULL;
+	static WMPropList *poplmenu = NULL;
 	int i;
 
 	if (data == NULL)
@@ -1467,6 +1535,7 @@ static WMPropList *processData(char *title, ItemData * data)
 	if (!pscut) {
 		pscut = WMCreatePLString("SHORTCUT");
 		pomenu = WMCreatePLString("OPEN_MENU");
+		poplmenu = WMCreatePLString("OPEN_PLMENU");
 	}
 
 	item = WMCreatePLArray(WMCreatePLString(title), NULL);
@@ -1522,9 +1591,14 @@ static WMPropList *processData(char *title, ItemData * data)
 		break;
 
 	case PipeInfo:
+	case PLPipeInfo:
 		if (!data->param.pipe.command)
 			return NULL;
-		WMAddToPLArray(item, pomenu);
+		if (data->type == PLPipeInfo)
+			WMAddToPLArray(item, poplmenu);
+		else
+			WMAddToPLArray(item, pomenu);
+
 		if (data->param.pipe.cached)
 			s1 = wstrconcat("| ", data->param.pipe.command);
 		else
