@@ -33,140 +33,28 @@
 #include <math.h>
 #include <time.h>
 
+#include <X11/XKBlib.h>
+
 #include <WINGs/WUtil.h>
 #include <wraster.h>
 
+#include "window.h"
+#include "misc.h"
 #include "WindowMaker.h"
 #include "GNUstep.h"
 #include "screen.h"
 #include "wcore.h"
 #include "window.h"
 #include "framewin.h"
-#include "funcs.h"
 #include "dialog.h"
 #include "xutil.h"
 #include "xmodifier.h"
 
-/**** global variables *****/
-extern WPreferences wPreferences;
 
-#ifdef USECPP
-static void putdef(char *line, char *name, char *value)
-{
-	if (!value) {
-		wwarning(_("could not define value for %s for cpp"), name);
-		return;
-	}
-	strcat(line, name);
-	strcat(line, value);
-}
+#define ICON_SIZE wPreferences.icon_size
 
-static void putidef(char *line, char *name, int value)
-{
-	char tmp[64];
-	snprintf(tmp, sizeof(tmp), "%i", value);
-	strcat(line, name);
-	strcat(line, tmp);
-}
-
-static char *username(void)
-{
-	char *tmp;
-
-	tmp = getlogin();
-	if (!tmp) {
-		struct passwd *user;
-
-		user = getpwuid(getuid());
-		if (!user) {
-			werror(_("could not get password entry for UID %i"), getuid());
-			return NULL;
-		}
-		if (!user->pw_name) {
-			return NULL;
-		} else {
-			return user->pw_name;
-		}
-	}
-	return tmp;
-}
-
-char *MakeCPPArgs(char *path)
-{
-	char buffer[MAXLINE], *buf, *line;
-	Visual *visual;
-	char *tmp;
-
-	line = wmalloc(MAXLINE);
-	*line = 0;
-	if ((buf = getenv("HOSTNAME")) != NULL) {
-		if (buf[0] == '(') {
-			wwarning(_("your machine is misconfigured. HOSTNAME is set to %s"), buf);
-		} else
-			putdef(line, " -DHOST=", buf);
-	} else if ((buf = getenv("HOST")) != NULL) {
-		if (buf[0] == '(') {
-			wwarning(_("your machine is misconfigured. HOST is set to %s"), buf);
-		} else
-			putdef(line, " -DHOST=", buf);
-	}
-	buf = username();
-	if (buf)
-		putdef(line, " -DUSER=", buf);
-	putidef(line, " -DUID=", getuid());
-	buf = XDisplayName(DisplayString(dpy));
-	putdef(line, " -DDISPLAY=", buf);
-	putdef(line, " -DWM_VERSION=", VERSION);
-
-	visual = DefaultVisual(dpy, DefaultScreen(dpy));
-	putidef(line, " -DVISUAL=", visual->class);
-
-	putidef(line, " -DDEPTH=", DefaultDepth(dpy, DefaultScreen(dpy)));
-
-	putidef(line, " -DSCR_WIDTH=", WidthOfScreen(DefaultScreenOfDisplay(dpy)));
-	putidef(line, " -DSCR_HEIGHT=", HeightOfScreen(DefaultScreenOfDisplay(dpy)));
-
-	/* put the dir where the menu is being read from to the
-	 * search path */
-	if (path) {
-		tmp = wstrdup(path);
-		buf = strchr(tmp + 1, ' ');
-		if (buf) {
-			*buf = 0;
-		}
-		buf = strrchr(tmp, '/');
-		if (buf) {
-			*buf = 0;	/* trunc filename */
-			putdef(line, " -I", tmp);
-		}
-		wfree(tmp);
-	}
-
-	/* this should be done just once, but it works this way */
-	strcpy(buffer, DEF_CONFIG_PATHS);
-	buf = strtok(buffer, ":");
-
-	do {
-		char fullpath[MAXLINE];
-
-		if (buf[0] != '~') {
-			strcpy(fullpath, buf);
-		} else {
-			/* home is statically allocated. Don't free it! */
-			char *home = wgethomedir();
-
-			strcpy(fullpath, home);
-			strcat(fullpath, &(buf[1]));
-		}
-
-		putdef(line, " -I", fullpath);
-
-	} while ((buf = strtok(NULL, ":")) != NULL);
-
-#undef arg
-	return line;
-}
-#endif				/* USECPP */
+/**** Local prototypes *****/
+static void UnescapeWM_CLASS(const char *str, char **name, char **class);
 
 /* XFetchName Wrapper */
 Bool wFetchName(Display *dpy, Window win, char **winname)
@@ -247,15 +135,38 @@ static void eatExpose(void)
 	}
 }
 
+void move_window(Window win, int from_x, int from_y, int to_x, int to_y)
+{
+#ifdef ANIMATIONS
+	if (wPreferences.no_animations)
+		XMoveWindow(dpy, win, to_x, to_y);
+	else
+		SlideWindow(win, from_x, from_y, to_x, to_y);
+#else
+	XMoveWindow(dpy, win, to_x, to_y);
+#endif
+}
+
 void SlideWindow(Window win, int from_x, int from_y, int to_x, int to_y)
+{
+	Window *wins[1] = { &win };
+	SlideWindows(wins, 1, from_x, from_y, to_x, to_y);
+}
+
+/* wins is an array of Window, sorted from left to right, the first is
+ * going to be moved from (from_x,from_y) to (to_x,to_y) and the
+ * following windows are going to be offset by (ICON_SIZE*i,0) */
+void SlideWindows(Window *wins[], int n, int from_x, int from_y, int to_x, int to_y)
 {
 	time_t time0 = time(NULL);
-	float dx, dy, x = from_x, y = from_y, sx, sy, px, py;
-	int dx_is_bigger = 0;
+	float dx, dy, x = from_x, y = from_y, px, py;
+	Bool is_dx_nul, is_dy_nul;
+	int dx_is_bigger = 0, dx_int, dy_int;
 	int slide_delay, slide_steps, slide_slowdown;
+	int i;
 
 	/* animation parameters */
-	static struct {
+	static const struct {
 		int delay;
 		int steps;
 		int slowdown;
@@ -271,12 +182,14 @@ void SlideWindow(Window win, int from_x, int from_y, int to_x, int to_y)
 	slide_steps = apars[(int)wPreferences.icon_slide_speed].steps;
 	slide_delay = apars[(int)wPreferences.icon_slide_speed].delay;
 
-	dx = (float)(to_x - from_x);
-	dy = (float)(to_y - from_y);
-	sx = (dx == 0 ? 0 : fabs(dx) / dx);
-	sy = (dy == 0 ? 0 : fabs(dy) / dy);
+	dx_int = to_x - from_x;
+	dy_int = to_y - from_y;
+	is_dx_nul = (dx_int == 0);
+	is_dy_nul = (dy_int == 0);
+	dx = (float) dx_int;
+	dy = (float) dy_int;
 
-	if (fabs(dx) > fabs(dy)) {
+	if (abs(dx_int) > abs(dy_int)) {
 		dx_is_bigger = 1;
 	}
 
@@ -286,17 +199,18 @@ void SlideWindow(Window win, int from_x, int from_y, int to_x, int to_y)
 			px = slide_steps;
 		else if (px > -slide_steps && px < 0)
 			px = -slide_steps;
-		py = (sx == 0 ? 0 : px * dy / dx);
+		py = (is_dx_nul ? 0.0 : px * dy / dx);
 	} else {
 		py = dy / slide_slowdown;
 		if (py < slide_steps && py > 0)
 			py = slide_steps;
 		else if (py > -slide_steps && py < 0)
 			py = -slide_steps;
-		px = (sy == 0 ? 0 : py * dx / dy);
+		px = (is_dy_nul ? 0.0 : py * dx / dy);
 	}
 
-	while (x != to_x || y != to_y) {
+	while (((int)x) != to_x ||
+			 ((int)y) != to_y) {
 		x += px;
 		y += py;
 		if ((px < 0 && (int)x < to_x) || (px > 0 && (int)x > to_x))
@@ -310,17 +224,19 @@ void SlideWindow(Window win, int from_x, int from_y, int to_x, int to_y)
 				px = slide_steps;
 			else if (px > -slide_steps && px < 0)
 				px = -slide_steps;
-			py = (sx == 0 ? 0 : px * dy / dx);
+			py = (is_dx_nul ? 0.0 : px * dy / dx);
 		} else {
 			py = py * (1.0 - 1 / (float)slide_slowdown);
 			if (py < slide_steps && py > 0)
 				py = slide_steps;
 			else if (py > -slide_steps && py < 0)
 				py = -slide_steps;
-			px = (sy == 0 ? 0 : py * dx / dy);
+			px = (is_dy_nul ? 0.0 : py * dx / dy);
 		}
 
-		XMoveWindow(dpy, win, (int)x, (int)y);
+		for (i = 0; i < n; i++) {
+			XMoveWindow(dpy, *wins[i], (int)x + i * ICON_SIZE, (int)y);
+		}
 		XFlush(dpy);
 		if (slide_delay > 0) {
 			wusleep(slide_delay * 1000L);
@@ -330,14 +246,16 @@ void SlideWindow(Window win, int from_x, int from_y, int to_x, int to_y)
 		if (time(NULL) - time0 > MAX_ANIMATION_TIME)
 			break;
 	}
-	XMoveWindow(dpy, win, to_x, to_y);
+	for (i = 0; i < n; i++) {
+		XMoveWindow(dpy, *wins[i], to_x + i * ICON_SIZE, to_y);
+	}
 
 	XSync(dpy, 0);
 	/* compress expose events */
 	eatExpose();
 }
 
-char *ShrinkString(WMFont * font, char *string, int width)
+char *ShrinkString(WMFont *font, const char *string, int width)
 {
 	int w, w1 = 0;
 	int p;
@@ -377,7 +295,7 @@ char *ShrinkString(WMFont * font, char *string, int width)
 	}
 	strcat(text, "...");
 	width -= WMWidthOfString(font, "...", 3);
-	pos = string;
+
 	p1 = 0;
 	p2 = p;
 	t = (p2 - p1) / 2;
@@ -397,7 +315,7 @@ char *ShrinkString(WMFont * font, char *string, int width)
 	return text;
 }
 
-char *FindImage(char *paths, char *file)
+char *FindImage(const char *paths, const char *file)
 {
 	char *tmp, *path = NULL;
 
@@ -516,7 +434,7 @@ static char *getselection(WScreen * scr)
 }
 
 static char*
-parseuserinputpart(char *line, int *ptr, char *endchars)
+parseuserinputpart(const char *line, int *ptr, const char *endchars)
 {
 	int depth = 0, begin;
 	char *value = NULL;
@@ -540,7 +458,7 @@ parseuserinputpart(char *line, int *ptr, char *endchars)
 }
 
 static char*
-getuserinput(WScreen *scr, char *line, int *ptr, Bool advanced)
+getuserinput(WScreen *scr, const char *line, int *ptr, Bool advanced)
 {
     char *ret = NULL, *title = NULL, *prompt = NULL, *name = NULL;
     int rv;
@@ -588,7 +506,7 @@ getuserinput(WScreen *scr, char *line, int *ptr, Bool advanced)
  * OPTION	etc.	NORMAL		%<input>
  */
 #define TMPBUFSIZE 64
-char *ExpandOptions(WScreen * scr, char *cmdline)
+char *ExpandOptions(WScreen *scr, const char *cmdline)
 {
 	int ptr, optr, state, len, olen;
 	char *out, *nout;
@@ -769,7 +687,7 @@ char *ExpandOptions(WScreen * scr, char *cmdline)
 	return NULL;
 }
 
-void ParseWindowName(WMPropList * value, char **winstance, char **wclass, char *where)
+void ParseWindowName(WMPropList *value, char **winstance, char **wclass, const char *where)
 {
 	char *name;
 
@@ -814,16 +732,16 @@ static char *keysymToString(KeySym keysym, unsigned int state)
 }
 #endif
 
-char *GetShortcutString(char *text)
+char *GetShortcutString(const char *shortcut)
 {
 	char *buffer = NULL;
 	char *k;
 	int modmask = 0;
 	/*    KeySym ksym; */
 	int control = 0;
-	char *tmp;
+	char *tmp, *text;
 
-	tmp = text = wstrdup(text);
+	tmp = text = wstrdup(shortcut);
 
 	/* get modifiers */
 	while ((k = strchr(text, '+')) != NULL) {
@@ -877,7 +795,26 @@ char *GetShortcutString(char *text)
 	return buffer;
 }
 
-char *EscapeWM_CLASS(char *name, char *class)
+char *GetShortcutKey(WShortKey key)
+{
+	char *tmp = NULL;
+	char *k = XKeysymToString(XkbKeycodeToKeysym(dpy, key.keycode, 0, 0));
+	if (!k) return NULL;
+
+	char **m = wPreferences.modifier_labels;
+	if (key.modifier & ControlMask) tmp = wstrappend(tmp, m[1] ? m[1] : "Ctrl+");
+	if (key.modifier & ShiftMask)   tmp = wstrappend(tmp, m[0] ? m[0] : "Shift+");
+	if (key.modifier & Mod1Mask)    tmp = wstrappend(tmp, m[2] ? m[2] : "Mod1+");
+	if (key.modifier & Mod2Mask)    tmp = wstrappend(tmp, m[3] ? m[3] : "Mod2+");
+	if (key.modifier & Mod3Mask)    tmp = wstrappend(tmp, m[4] ? m[4] : "Mod3+");
+	if (key.modifier & Mod4Mask)    tmp = wstrappend(tmp, m[5] ? m[5] : "Mod4+");
+	if (key.modifier & Mod5Mask)    tmp = wstrappend(tmp, m[6] ? m[6] : "Mod5+");
+	tmp = wstrappend(tmp, k);
+
+	return GetShortcutString(tmp);
+}
+
+char *EscapeWM_CLASS(const char *name, const char *class)
 {
 	char *ret;
 	char *ename = NULL, *eclass = NULL;
@@ -932,7 +869,7 @@ char *EscapeWM_CLASS(char *name, char *class)
 	return ret;
 }
 
-void UnescapeWM_CLASS(char *str, char **name, char **class)
+static void UnescapeWM_CLASS(const char *str, char **name, char **class)
 {
 	int i, j, k, dot;
 
@@ -983,7 +920,7 @@ void UnescapeWM_CLASS(char *str, char **name, char **class)
 	}
 }
 
-void SendHelperMessage(WScreen * scr, char type, int workspace, char *msg)
+void SendHelperMessage(WScreen *scr, char type, int workspace, const char *msg)
 {
 	char *buffer;
 	int len;
@@ -1048,7 +985,7 @@ Bool UpdateDomainFile(WDDomain * domain)
 	return result;
 }
 
-char *StrConcatDot(char *a, char *b)
+char *StrConcatDot(const char *a, const char *b)
 {
 	int len;
 	char *str;
@@ -1094,9 +1031,3 @@ char *GetCommandForWindow(Window win)
 {
 	return getCommandForWindow(win, 0);
 }
-
-/* Free result when done */
-char *GetProgramNameForWindow(Window win)
-{
-	return getCommandForWindow(win, 1);
-}
